import pyvizionsdk
from pyvizionsdk import VX_CAMERA_INTERFACE_TYPE, VX_IMAGE_FORMAT, VX_ISP_IMAGE_PROPERTIES, VX_OSP_PROFILE_FLAG

import inquirer
from inquirer.themes import BlueComposure
import os
import numpy as np

class pyVizionCtl:
    def clearConsole(self):
        return os.system("cls" if os.name in ("nt", "dos") else "clear")

    def GetDeviceList(self):
        result, camera_list = pyvizionsdk.VxDiscoverCameraDevices()
        cameras = []
        for camera in camera_list:
            cameras.append(camera)

        if (len(cameras) == 0):
            self.clearConsole()
            options = [
                inquirer.List('pyVizionCtl',
                                message="No Camera detected! Please exit.",
                                choices = ["exit"],
                                carousel = True),
            ]
            entryIndex = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
            if (entryIndex == "exit"):
                exit()
        else:
            self.clearConsole()
            options = [
                inquirer.List('pyVizionCtl',
                                message="Cameras List",
                                choices = cameras,
                                carousel = True),
            ]
            dev = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
            devIndex = cameras.index(dev)
            cam = self.InitialDev(devIndex)
            self.CtlList(cam)

    
    def InitialDev(self, devIndex):
        # initial the device and open the camera
        cam = pyvizionsdk.VxInitialCameraDevice(devIndex)
        pyvizionsdk.VxOpen(cam)
        return cam

    def CtlList(self, cam):
        self.clearConsole()
        options = [
            inquirer.List('pyVizionCtl',
                            message="Camera Control and Management Tool",
                            choices = ["Device Information", "Camera Control", 
                                        "List Formats", "Image operations", 
                                        "OSP Profile management", "Timestamp operations", 
                                        " < Back to Device List", "exit"],
                            carousel = True),
        ]
        ctlIndex = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        #Device Information
        if (ctlIndex == "Device Information"):
            return self.DeviceInfo(cam)
        # Camera Control
        elif (ctlIndex == "Camera Control"): 
            return self.CameraControl(cam)
        # List Formats
        elif (ctlIndex == "List Formats"): 
            return self.FormatList(cam)
        # Image operations
        elif (ctlIndex == "Image operations"):
            return self.ImageOperation(cam)
        # OSP Profile management
        elif (ctlIndex == "OSP Profile management"):
            return self.OSPProfile(cam)
        # Timestamp operations
        elif (ctlIndex == "Timestamp operations"):
            return self.Timestamp(cam)
        # Back to Device List
        elif (ctlIndex == " < Back to Device List"):
            pyvizionsdk.VxClose(cam)
            return self.GetDeviceList()
        # exit
        elif (ctlIndex == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def DeviceInfo(self, cam):
        self.clearConsole()
        print("Device Information:")
        _, devName = pyvizionsdk.VxGetDeviceName(cam)
        print("Device Name: ", devName)
        
        # get information
        if (pyvizionsdk.VxIsVizionCamera(cam) == 0):
            # get interface type
            _, tyname = pyvizionsdk.VxGetDeviceInterfaceType(cam)
            if (tyname == VX_CAMERA_INTERFACE_TYPE.INTERFACE_USB):
                typeStr = "USB"
                _, usbFwVer = pyvizionsdk.VxGetUSBFirmwareVersion(cam)
                _, hwId = pyvizionsdk.VxGetHardwareID(cam)
                print("HardWareID: ", hwId)
                print("Interface Type: ", typeStr)
                print("USB Firmware: ", usbFwVer)
            elif (tyname == VX_CAMERA_INTERFACE_TYPE.INTERFACE_MIPI_CSI2):
                typeStr = "Sensor"
                print("Interface Type: ", typeStr)
            else:
                typeStr = "Unknown"
                print("Interface Type: ", typeStr)
        
        # device information menu
        options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Camera Control and Management Tool", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # select other control
        if (index == " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def CameraControl(self, cam):
        controls = {"Brightness" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_BRIGHTNESS,
                    "Contrast" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_CONTRAST,
                    "Saturation" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_SATURATION,
                    "Gamma" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_GAMMA,
                    "Sharpness" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_SHARPNESS,
                    "Backlight" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_BACKLIGHT_COMPENSATION,
                    "Noise" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_DENOISE,
                    "White_Balance_Mode" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_WHITEBALANCE_MODE,
                    "White_Balance_Temperature" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_WHITEBALANCE_TEMPERATURE,
                    "JPEG_Quality" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_JPEG_QUALITY,
                    "Exposure_Mode" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_EXPOSURE_MODE,
                    "Exposure_Time" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_EXPOSURE_TIME,
                    "Exposure_Min_Time" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_EXPOSURE_MIN_TIME,
                    "Exposure_Max_Time" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_EXPOSURE_MAX_TIME,
                    "Exposure_Gain" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_EXPOSURE_GAIN,
                    "Flick_Mode" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_FLICK_MODE,
                    "Special_Effect" :  VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_SPECIAL_EFFECT_MODE,
                    "Flip_Mode" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_FLIP_MODE,
                    "Pan_Target" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_PAN,
                    "Tilt_Target" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_TILT,
                    "Zoom_Target" : VX_ISP_IMAGE_PROPERTIES.ISP_IMAGE_ZOOM,
                    "Throughput" : pyvizionsdk.VxGetThroughPut(cam),
                    "Max_FPS" : pyvizionsdk.VxGetMaxFPS(cam)
        }

        self.clearConsole()
        options = [
            inquirer.List('pyVizionCtl',
                            message = "Camera Control",
                            choices = ["Get Controls", "Set Controls", " < Back to Camera Control and Management Tool", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # get controls
        if (index == "Get Controls"):
            return self.GetControl(cam, controls)
        # set controls
        elif (index == "Set Controls"):
            return self.SetControl(cam, controls)
        # select other control
        elif (index == " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()
    
    def GetControl(self, cam, controls):
        self.clearConsole()
        options = [
            inquirer.List('pyVizionCtl',
                            message = "Get Controls",
                            choices = ["Get all controls value", "Get specific control value", " < Back to Camera Control Menu", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
        
        # Get all controls
        if (index ==  "Get all controls value"):
            self.GetAllControl(cam, controls)
            options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Get Controls Menu", " << Back to Camera Control Menu", "exit"],
                            carousel = True),
            ]
            index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
            
            # Back to Get Control Menu
            if (index ==  " < Back to Get Controls Menu"):
                return self.GetControl(cam, controls)
            # Back to Camera Control Menu
            elif (index == " << Back to Camera Control Menu"):
                return self.CameraControl(cam)
            # exit
            elif (index == "exit"):
                # close the camera when exit
                self.clearConsole()
                pyvizionsdk.VxClose(cam)
                exit()
        # Get specific control
        elif (index ==  "Get specific control value"):
            return self.GetSpecificControl(cam, controls)
        # Back to Camera Control Menu
        elif (index == " < Back to Camera Control Menu"):
            return self.CameraControl(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()
    
    def GetAllControl(self, cam, controls):
        self.clearConsole()
        print("Available controls:")
        print("=============================================================")
        for enum, (propName, propId) in enumerate(controls.items()):
            if (propName == "Throughput"):
                throughput = propId[1]
                print(f"[{enum:2}] {propName:25} {throughput:>8} | {10.0:>8.2f} - {1000.0:<8.2f}")
            elif (propName == "Max_FPS"):
                maxFPS = propId[1]
                print(f"[{enum:2}] {propName:25} {maxFPS:>8} | {1:>8} - {120:<8}")
            else:
                value, flag = pyvizionsdk.VxGetISPImageProcessing(cam, propId)[1:]
                minVal, maxVal = pyvizionsdk.VxGetISPImageProcessingRange(cam, propId)[1:3]
                if (flag == 1):
                    print(f"{propName} control is read-only")
                    print(f"[{enum:2}] {propName:25} {value:>8} | {minVal:>8} - {maxVal:<8} (Read-only)")
                else:
                    print(f"[{enum:2}] {propName:25} {value:>8} | {minVal:>8} - {maxVal:<8}")

            if (propName == "White_Balance_Mode"):
                print(f"{'':39} | {'':>6} Manual Temperature(0), Auto(1)")
            elif(propName == "Exposure_Mode"):
                print(f"{'':39} | {'':>6} Manual Mode(0), Auto Mode(1), Auto gain(2)")
            elif(propName == "Flick_Mode"):
                print(f"{'':39} | {'':>6} Disable(0), 50Hz(1), 60Hz(2), Auto(3)")
            elif(propName == "Special_Effect"):
                print(f"{'':39} | {'':>6} Normal Mode(0), Black White Mode(1), Grayscale Mode(2), Negative Mode(3), Sketch Mode(4)")
            elif(propName == "Flip_Mode"):
                print(f"{'':39} | {'':>6} Normal(0), H-Mirror(1), V-Mirror(2), Rotate-180(3)")
    
    def GetSpecificControl(self, cam, controls):          
        self.clearConsole()

        for enum, prop in enumerate(controls):
            print(f"[{enum}] {prop}")
        propIndex = input("\nEnter a property index: ")
        propIndex = int(propIndex)
        
        if propIndex < len(controls):
            propName = list(controls)[propIndex]
            propId = list(controls.values())[propIndex]
            print("{:25} {:>8} | {:>8}".format("Name", "Value", "Range"))
            if (propName == "Throughput"):
                throughput = propId[1]
                print(f"{propName:25} {throughput:>8} | {10.0:>8.2f} - {1000.0:<8.2f}")
            elif (propName == "Max_FPS"):
                maxFPS = propId[1]
                print(f"{propName:25} {maxFPS:>8} | {1:>8} - {120:<8}")
            else:
                value, flag = pyvizionsdk.VxGetISPImageProcessing(cam, propId)[1:]
                minVal, maxVal = pyvizionsdk.VxGetISPImageProcessingRange(cam, propId)[1:3]
                if (flag == 1):
                    print(f"{propName} control is read-only")
                    print(f"{propName:25} {value:>8} | {minVal:>8} - {maxVal:<8} (Read-only)")
                else:
                    print(f"{propName:25} {value:>8} | {minVal:>8} - {maxVal:<8}")
        else:
            self.clearConsole()
            print("The property index is out of range.")
            self.GetSpecificControl(cam, controls)

        options = [
        inquirer.List('pyVizionCtl',
                        choices = [" < Back to Get Controls Menu", " << Back to Camera Control Menu", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
        
        # Back to Get Control Menu
        if (index ==  " < Back to Get Controls Menu"):
            return self.GetControl(cam, controls)
        # Back to Camera Control Menu
        elif (index == " << Back to Camera Control Menu"):
            return self.CameraControl(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def SetControl(self, cam, controls):
        self.GetAllControl(cam, controls)
        propIndex = input("\nEnter a property index: ")
        propIndex = int(propIndex)
        propName = list(controls)[propIndex]

        if (propIndex > len(controls) - 1):
            self.clearConsole()
            print("The property is out of index!")
            return self.SetControl(cam, controls)
        elif (propName != "Throughput" and propName != "Max_FPS"):
            propName = list(controls)[propIndex]
            propId = list(controls.values())[propIndex]
            value, flag = pyvizionsdk.VxGetISPImageProcessing(cam, propId)[1:]
            if (flag == 1):
                self.clearConsole()
                print(f"{propName} control is read-only")
                return self.SetControl(cam, controls)

        propName = list(controls)[propIndex]
        propId = list(controls.values())[propIndex]

        if (propName == "Throughput"):
            minVal = 10.00
            maxVal = 1000.00
        elif (propName == "Max_FPS"):
            minVal = 1
            maxVal = 120
        else:
            minVal, maxVal = pyvizionsdk.VxGetISPImageProcessingRange(cam, propId)[1:3]

        value = input(f"\n(Range of {propName}: {minVal}-{maxVal})\nEnter a value: ")
        value = int(value)

        while value < minVal or value > maxVal:
            print(f"The value is out of range. {propName} Range: {minVal}-{maxVal}")
            value = input("\nEnter a value: ")
            value = int(value)
        
        if (propName == "Throughput"):
            pyvizionsdk.VxSetThroughPut(cam, value)
        elif (propName == "Max_FPS"):
            pyvizionsdk.VxSetMaxFPS(cam, value)
        else:
            pyvizionsdk.VxSetISPImageProcessing(cam, propId, value)
        
        print("Set Successfully!")
        options = [
        inquirer.List('pyVizionCtl',
                        choices = [" < Back to Camera Control Menu", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
        
        # Back to Camera Control Menu
        if (index == " < Back to Camera Control Menu"):
            return self.CameraControl(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def FormatList(self, cam):
        self.clearConsole()
        formatList, formats = self.GetFormatList(cam)
        options = [
        inquirer.List('pyVizionCtl',
                        message = "Format List",
                        choices = ["Set format", " < Back to Camera Control and Management Tool", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
        
        # Back to set format
        if (index ==  "Set format"):
            if(self.SetFormat(cam, formatList, formats)[0] == 0):
                options = [
                inquirer.List('pyVizionCtl',
                                choices = [" << Back to Camera Control and Management Tool", "exit"],
                                carousel = True),
                ]
                index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
                # Back to Format List
                if (index == " << Back to Camera Control and Management Tool"):
                    return self.CtlList(cam)
                # exit
                elif (index == "exit"):
                    # close the camera when exit
                    self.clearConsole()
                    pyvizionsdk.VxClose(cam)
                    exit()
            
        # Back to Camera Control and Management Tool
        elif (index == " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()
    
    def GetFormatList(self, cam):
        # display th format list
        result, formatList = pyvizionsdk.VxGetFormatList(cam)
        formats =[]
        for enum, format in enumerate(formatList):
            formatStr = f"[{enum}]: Resolution: {format.width} x {format.height}\n" +\
                        f"Frame Rate: {format.framerate}\n" +\
                        f"Format: {format.format}"
            print(formatStr)
            formats.append(formatStr)
        return formatList, formats


    def SetFormat(self, cam, formatList, formats):
        formatIndex = input("\nChoose a format index: ")
        index = int(formatIndex)

        if (index < len(formatList)):
            self.clearConsole()
            pyvizionsdk.VxSetFormat(cam, formatList[index])
            print("Set Format Success: \n", formats[index])
        else:
            self.clearConsole()
            for f in formats:
                print(f)
            print("Out of format list index!")
            return self.SetFormat(cam, formatList, formats)
        return 0, formatList[index]

    def ImageOperation(self, cam):
        self.clearConsole()
        options = [
        inquirer.List('pyVizionCtl',
                        message = "Image Operation",
                        choices = ["Save Image", " < Back to Camera Control and Management Tool", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # save image
        if (index ==  "Save Image"):
            return self.SaveImage(cam)
        # select other control
        elif (index ==  " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def SaveImage(self, cam):
        self.clearConsole()
        filePath = input("Enter the File Path (e.g: test.raw): ")
        formatList, formats = self.GetFormatList(cam)
        result, imgFormat = self.SetFormat(cam, formatList, formats)
        
        imgSkip = int(input("Enter the numbers of image to skip: "))
        while (imgSkip < 0):
            print("Number is invalid, please enter the number >= 0.")
            imgSkip = int(input("Enter the numbers of image to skip: "))
        
        imgSave = int(input("Enter the numbers of image to save: "))
        while (imgSave <= 0):
            print("Number is invalid, please enter the number > 0.")
            imgSave = int(input("Enter the numbers of image to save: "))

        pyvizionsdk.VxStartStreaming(cam)

        for s in range(1, imgSkip + 1):
            print("Skip ", s)
            result, imgArr= pyvizionsdk.VxGetImage(cam, 3000, imgFormat)

        for i in range(1, imgSave + 1):
            if (imgSave > 1):
                fileName, exetension = filePath.rsplit('.', 1)
                newfilePath = fileName + '_' + str(i) + '.' + exetension
                result, imgArr= pyvizionsdk.VxGetImage(cam, 3000, imgFormat)
                imgArr.tofile(newfilePath)
                print(f"Save the image to {newfilePath}")
            else:
                result, imgArr= pyvizionsdk.VxGetImage(cam, 3000, imgFormat)
                imgArr.tofile(filePath)
                print(f"Save the image to {filePath}")
                
        pyvizionsdk.VxStopStreaming(cam)
        
        options = [
        inquirer.List('pyVizionCtl',
                        choices = [" << Back to Camera Control and Management Tool", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # select other control
        if (index ==  " << Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def OSPProfile(self, cam):
        flagType = {VX_OSP_PROFILE_FLAG.DISABLED : "DISABLED",
                    VX_OSP_PROFILE_FLAG.ENABLED : "ENABLED",
                    VX_OSP_PROFILE_FLAG.ENABLED_AND_SAVE : "ENABLE_AND_SAVE"
        }
        self.clearConsole()
        options = [
        inquirer.List('pyVizionCtl',
                        message = "OSP Profile Management",
                        choices = ["Get OSP Flag", "Set OSP Flag", " < Back to Camera Control and Management Tool", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # get OSP flag
        if (index ==  "Get OSP Flag"):
            return self.GetOSPFlag(cam, flagType)
        # set OSP flag
        elif (index == "Set OSP Flag"):
            return self.SetOSPFlag(cam, flagType)
        # select other control
        elif (index == " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def GetOSPFlag(self, cam, flagType):
        self.clearConsole()
        flag = pyvizionsdk.VxGetOSPProfileFlag(cam)[1]
        flagStr = flagType[flag]
        print("OSP Profile Flag: ", flagStr)

        options = [
        inquirer.List('pyVizionCtl',
                        message = "OSP Profile Management",
                        choices = [" < Back to OSP Profile Management", "exit"],
                        carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # Back to OSP Profile Management
        if (index == " < Back to OSP Profile Management"):
            return self.OSPProfile(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def SetOSPFlag(self, cam, flagType):
        self.clearConsole()
        options = [
            inquirer.List('setOSP',
                            choices = [option for option in flagType.values()],
                            carousel = True),
        ]
        ftype = inquirer.prompt(options, theme=BlueComposure())['setOSP']
        flagIndex = (list(flagType.values())).index(ftype)
        # DISABLED ENABLED ENABLED_AND_SAVE
        pyvizionsdk.VxSetOSPProfileFlag(cam, list(flagType)[flagIndex])
        print("Set OSP Profile Successful!")

        ospOptions = [
            inquirer.List('pyVizionCtl',
                            message = "OSP Profile Management",
                            choices = [" < Back to OSP Profile Management", "exit"],
                            carousel = True),
        ]
        ospIndex = inquirer.prompt(ospOptions, theme=BlueComposure())['pyVizionCtl']
        # Back to OSP Profile Management
        if (ospIndex == " < Back to OSP Profile Management"):
            return self.OSPProfile(cam)
        # exit
        elif (ospIndex == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def Timestamp(self, cam):
        self.clearConsole()
        options = [
            inquirer.List('pyVizionCtl',
                            message = "Timestamp Operations:(The reset functions must be used when the camera is streaming.)",
                            choices = ["Get Timestamp", "Get FrameCount", 
                                        "Reset Timestamp", "Reset FrameCount", 
                                        " < Back to Camera Control and Management Tool", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # get timestamp
        if (index ==  "Get Timestamp"):
            return self.GetTimestamp(cam)
        # get framecount
        elif (index == "Get FrameCount"):
            return self.GetFrameCount(cam)
        # reset timestamp
        elif (index == "Reset Timestamp"):
            return self.ResetTimestamp(cam)
        # reset framecount
        elif (index == "Reset FrameCount"):
            return self.ResetFrameCount(cam)
        # select other control
        elif (index == " < Back to Camera Control and Management Tool"):
            return self.CtlList(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()
    
    def GetTimestamp(self, cam):
        self.clearConsole()
        timestamp = pyvizionsdk.VxGetTimestamp(cam)[1]
        print(f"Timestamp: {timestamp}\n")

        options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Timestamp Operations", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # Back to Timestamp Operations
        if (index ==  " < Back to Timestamp Operations"):
            return self.Timestamp(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()
        

    def GetFrameCount(self, cam):
        self.clearConsole()
        frameCount = pyvizionsdk.VxGetFrameCount(cam)[1]
        print(f"FrameCount: {frameCount}\n")

        options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Timestamp Operations", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # Back to Timestamp Operations
        if (index ==  " < Back to Timestamp Operations"):
            return self.Timestamp(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def ResetTimestamp(self, cam):
        self.clearConsole()
        pyvizionsdk.VxResetTimestamp(cam)

        options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Timestamp Operations", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # Back to Timestamp Operations
        if (index ==  " < Back to Timestamp Operations"):
            return self.Timestamp(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

    def ResetFrameCount(self, cam):
        self.clearConsole()
        pyvizionsdk.VxResetFrameCount(cam)

        options = [
            inquirer.List('pyVizionCtl',
                            choices = [" < Back to Timestamp Operations", "exit"],
                            carousel = True),
        ]
        index = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']

        # Back to Timestamp Operations
        if (index ==  " < Back to Timestamp Operations"):
            return self.Timestamp(cam)
        # exit
        elif (index == "exit"):
            # close the camera when exit
            self.clearConsole()
            pyvizionsdk.VxClose(cam)
            exit()

def main():
    pyCtl = pyVizionCtl()
    pyCtl.clearConsole()
    options = [
        inquirer.List('pyVizionCtl',
                        message="pyVizionCtl",
                        choices = ["Device List", "exit"],
                        carousel = True),
    ]

    entryIndex = inquirer.prompt(options, theme=BlueComposure())['pyVizionCtl']
    if (entryIndex == "Device List"):
        pyCtl.GetDeviceList()
    elif (entryIndex == "exit"):
        exit()

if __name__ == '__main__':
    main()
        